// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "DawnPokerGame.hpp"
#include "asset/assets/TextureAsset.hpp"

using namespace Dawn;

std::shared_ptr<TrueTypeAsset> asset;
std::shared_ptr<TextureAsset> assetTexture;
std::shared_ptr<UILabel> label;
float_t fs = 1.0f;

DawnGame::DawnGame(DawnHost &host) :
  host(host),
  renderManager(*this)
{
}

int32_t DawnGame::init() {
  this->assetManager.init();
  this->renderManager.init();

  this->scene = std::make_shared<Scene>(*this);

  auto cameraObject = this->scene->createSceneItem();
  auto camera = cameraObject->addComponent<Camera>();
  camera->transform.lookAt(glm::vec3(50, 50, 50), glm::vec3(0, 0, 0));
  
  auto canvas = UICanvas::createCanvas(this->scene);
  
  asset = this->assetManager.load<TrueTypeAsset>("truetype_ark");
  assetTexture = this->assetManager.load<TextureAsset>("texture_test");
  while(!asset->loaded || !assetTexture->loaded) {
    this->assetManager.update();
  }

  // auto sprite = canvas->addElement<UISprite>();
  // sprite->setTransform(
  //   UI_COMPONENT_ALIGN_START,
  //   UI_COMPONENT_ALIGN_START,
  //   glm::vec4(0, 0, 200, 200),
  //   0
  // );
  // sprite->texture = &asset->font.getTexture();


  label = canvas->addElement<UILabel>();
  label->setTransform(
    UI_COMPONENT_ALIGN_START,
    UI_COMPONENT_ALIGN_START,
    glm::vec4(0, 0, 200, 200),
    0
  );
  label->setText("Hello World\nHow are you today?");
  label->setFont(&asset->font);

  return DAWN_GAME_INIT_RESULT_SUCCESS;
}

int32_t DawnGame::update(float_t delta) {
  this->assetManager.update();

  if(this->scene != nullptr) this->scene->update();

  label->setFontSize(fs += delta);
  
  this->renderManager.update();

  return DAWN_GAME_UPDATE_RESULT_SUCCESS;
}

DawnGame::~DawnGame() {
  
}